--[[Snake 2.0 on Love2D Framework- Chris Harris, Lindsay Hill, Jeffrey Wellman, Ethan Ramsey
-- Create a 4:8 window that scales to the screen the game starts on, in a windowed mode that does not require focus to be playing 
-- Have a main menu, with the options of Play, Quit, and Highscores, with a hidden menu for the AI auto play mode 
-- If the player hits F11 they will toggle / untoggle fullscreen mode (OPTIONAL)
-- The AI Auto play mode only unlocks when the player has hit at least 300 points 
-- The snake will be outlined in a neon color 
-- Make sure there is some form of wall for the player to see 
-- Snake grid will be 25x25 
-- Every 10 points a different sound will play 
-- When the player dies they will be redirected to the highscore screen only if they have something to enter --]]
--8 bit colors require 8-8-4 rather than 255-255-255
--this means 0.003921568627 * (1-8/4) will be your output, white being 0.314,0.314,0.157
font = love.graphics.newFont("p2start.ttf",16)
small_font = love.graphics.newFont("p2start.ttf",12)
love.graphics.setFont(font)
ss = require("simplescale") -- aspect ratio library 
moonshine = require("moonshine") -- shader library 
snake = require("snake") -- snake game library
res = require("resources") -- debug library
love.filesystem.setIdentity("Snake")
function love.load() 
	if pcall(require, "jit.opt") then
  		require("jit.opt").start(
    	"sizemcode=64", -- this is the default on windows, but I'm just enforcing it
    	"maxmcode=8192", --previously in löve 11.3 this was 8192
    	"maxtrace=8192"  --previously in löve 11.3 this was 8192
  		)
	end
	delta_time = 0
	debugging = false 
	local _, _, flags = love.window.getMode()
	local width,height = love.window.getDesktopDimensions(flags.display)
	background = love.graphics.newImage("sprites/background.png")
	ss.setWindow(960,960,width,height,flags) -- all of this sets the window to the size of your desktop in a windowed fullscreen mode, while maintaining the aspect ratio of the game

	effect = moonshine(960,960,moonshine.effects.crt) -- custom shader 
			.chain(moonshine.effects.scanlines)
	effect.scanlines.thickness = 0.325
	effect.scanlines.width = 2.4
	effect_counter = 0 -- counter for scanlines 

	screen = love.graphics.newCanvas(960,960)
	snake.load()
	snake.paused = true
	counter = 0 -- when reaches 15, update snake 
	counter_max = 1.9
end
function love.keypressed(key)
	snake.keypressed(key)
	if string.lower(key) == "p" then snake.paused = not snake.paused end
	if key == "printscreen" then 
		love.graphics.captureScreenshot(os.time()..'.png')
	elseif key == "f1" then 
		debugging = not debugging
	end
end
function bps_frombpm(bpm)
	local s = bpm/60
	return s -- seconds 
end
function love.update(dt)
	delta_time = dt
	--counter_max updater
	counter_max = 1/(math.floor((snake.score/50) + 1) * 1.9)
	--
	--scanline updater
	effect_counter = effect_counter + 0.12
	if effect_counter >= 180 then 
		effect_counter = 0 
	end
	effect.scanlines.phase = effect_counter
	--
	--snake update counter
	if snake.paused == false then
		if snake.direction == nil then 
			snake.paused = true 
		else 
			counter = counter + delta_time
			if counter >= counter_max then -- game updates at 2x per second, but when the player reaches 250 score it should be at 4x per second  
				counter = 0 
				snake.update()
				--collectgarbage("collect")
			end
		end
	end
	
	--
	screen:renderTo(function()
		effect(function()
			love.graphics.setColor(1,1,1)
			love.graphics.draw(background,0,0)
			snake.draw()
			if debugging then res.draw() require("lovebird").update() end 
		end)
	end)
end
function love.draw()
	love.graphics.clear()
	ss.set() -- enabling the aspect ratio calculations 
	love.graphics.draw(screen)
	ss.unSet()
	if debugging then res.update(delta_time) end 
end