local snake = {} -- the snake game library 
snake.paused = false -- whether the game is paused
snake.ding = love.audio.newSource("sound/ding.mp3","static")
snake.score_sound = love.audio.newSource("sound/score.mp3","static")
snake.die = love.audio.newSource("sound/die.mp3", "static")
snake.background = love.audio.newSource("sound/background_music.mp3","stream")
snake.sprites = {} 
snake.sprites.head = love.graphics.newImage("sprites/head.png")
snake.sprites.body = love.graphics.newImage("sprites/body.png")
snake.sprites.food = love.graphics.newImage("sprites/food.png")
function snake.dto_r(d) -- direction to rotation 
	--default position is up
	local r = 0
	if d == "up" then 
	elseif d == "left" then 
		r = -90
	elseif d == "right" then 
		r = 90
	elseif d == "down" then 
		r = 180
	end
	return math.rad(r)
end
font = love.graphics.newFont("p2start.ttf",16)
function snake.load()
	sound_counter = 0 -- for ding/score
	cell_size = {960/25,900/25} -- size of each cell in pixels for drawing
	snake.direction = nil -- direction the snake is traveling 
	snake.all_positions = {} --all positions of the snake
	snake.score = 0 -- player score 
	snake.grid = {} -- game grid 
	for x = 1,25 do 
		snake.grid[x] = {}
		for y = 1,25 do 
			snake.grid[x][y] = {0} -- default cell value is 0, meaning background, 1 meaning snake, 2 meaning food 
		end
	end
	-- starting snake/food positions 
	local food_pos = {0,0}
	local snake_pos = {0,0} 
	repeat 
		food_pos = {math.random(1,25),math.random(1,25)}
		snake_pos = {math.random(1,25),math.random(1,25)}
	until food_pos[1] ~= snake_pos[1] and food_pos[2] ~= snake_pos[2]
	snake.grid[food_pos[1]][food_pos[2]] = 2 
	snake.grid[snake_pos[1]][snake_pos[2]] = 1 
	--
	--remember, if direction is nil, game will be set to paused until the player assigns a direction, or hits P in which case an optimal direction will be assigned. 
	if snake_pos[1] > snake_pos[2] then -- left or right 
		if snake_pos[1] >= 25 then -- left 
			snake.direction = "left"
		else
			snake.direction = "right"
		end
	else--up or down 
		if snake_pos[2] >= 25 then -- up
			snake.direction = "up"
		else
			snake.direction = "down"
		end
	end
	local snake_cell = {}
	snake_cell.x = snake_pos[1]
	snake_cell.y = snake_pos[2]
	snake_cell.direction = snake.direction
	table.insert(snake.all_positions,snake_cell) -- insert the snake cell into the table so it can be kept track of 
	snake.background:setVolume(0.75)
	snake.background:setLooping(true)
	snake.background:play()
end
function snake.keypressed(key) -- assign direction on appropriate key press 
	if string.lower(key) == "w" or key == "up" then 
		snake.direction = "up"
	elseif string.lower(key) == "a" or key == "left" then 
		snake.direction = "left"
	elseif string.lower(key) == "d" or key == "right" then 
		snake.direction = "right"
	elseif string.lower(key) == "s" or key == "down" then 
		snake.direction = "down"
	end
end
function snake.in_bounds(x,y) -- returns whether the player is in bounds or not 
	local in_b = false
	if x >= 1 and x <= 25 then 
		if y >= 1 and y <= 25 then 
			in_b = true 
		end
	end
	return in_b
end
function snake.update() -- calculate next frame 
	--first, get the X and Y of the position you would potentially be moving to 
	local alive = true 
	local scored = false
	local head = snake.all_positions[#snake.all_positions]
	local moving_to = {head.x,head.y}
	if snake.direction == "up" then 
		moving_to[2] = moving_to[2] - 1 
	elseif snake.direction == "down" then 
		moving_to[2] = moving_to[2] + 1 
	elseif snake.direction == "left" then
		moving_to[1] = moving_to[1] - 1 
	elseif snake.direction == "right" then
		moving_to[1] = moving_to[1] + 1 
	end
	--next check if this is in bounds, if not do nothing and change the alive variable to false 
	if snake.in_bounds(moving_to[1],moving_to[2]) == true then 
	--if in bounds, check what currently occupies that actual cell in the grid. 
		local cell = snake.grid[moving_to[1]][moving_to[2]]
		if cell == 0 then --if its a background piece, you're good, 
		elseif cell == 2 then --if its a food piece, note the player has scored,
			scored = true 
		elseif cell == 1 then 
			alive = false 
		end--if its a snake piece, the player is no longer alive 
	elseif snake.in_bounds(moving_to[1],moving_to[2]) == false then 
		alive = false
	end
	if alive == true then --check if the player is still alive 
		if scored ~= true then --check if they have scored
			--if they have scored, add 5 to the score (This order has been reversed)
			snake.grid[snake.all_positions[1].x][snake.all_positions[1].y] = 0
			table.remove(snake.all_positions,1)
		else--if they havent scored, remove the first cell of snake.all_positions
			snake.score = snake.score + 5 
			sound_counter = sound_counter + 1 
			if sound_counter >= 5 then 
				sound_counter = 0 
				snake.ding:stop()
				snake.ding:play()
			else
				snake.score_sound:stop()
				snake.score_sound:play()
			end
			--spawn new food
			local newfood_position = {math.random(1,25),math.random(1,25)}
			local occupied = false
			repeat 
			newfood_position = {math.random(1,25),math.random(1,25)}
			for i,v in pairs(snake.all_positions) do 
				if v.x == newfood_position[1] and v.y == newfood_position[2] then 
					occupied = true 
				end
			end
			until occupied == false  
			snake.grid[newfood_position[1]][newfood_position[2]] = 2 
			--
		end
		snake.grid[moving_to[1]][moving_to[2]] = 1 --if the player is alive, change the grid cell they would have moved to, to a snake cell 
		local snake_cell = {}
		snake_cell.x = moving_to[1]
		snake_cell.y = moving_to[2]
		snake_cell.direction = snake.direction
		table.insert(snake.all_positions,snake_cell)
	else--if the player is not alive, call the load function and pause the game
		snake.die:stop()
		snake.die:play()
		snake.load()
		snake.paused = true
	end
	scored = false 
end
function snake.draw()
	love.graphics.setFont(font)
	--the game window is 960x960 pixels, the top scoreboard is going to be 60 pixels. So display the game grid at y60, and at 960x900. 
	love.graphics.setColor(0.4,0.4,0.4)
	love.graphics.rectangle("fill",0,0,960,60) -- scoreboard rectangle
	love.graphics.setColor(0,0,0)
	love.graphics.print("SCORE:"..tostring(snake.score),5,15)
	if snake.paused then love.graphics.print("PAUSED",5,40) end 
	for x = 1,25 do 
		for y = 1,25 do -- draw the game grid 
			local pos = {((x-1) * cell_size[1])+19.2, ((y-1) * cell_size[2])+79.2}
			local v = snake.grid[x][y] -- current grid cell 
			if v == 0 then --if background draw nothing
			elseif v == 1 then -- if snake draw blue dot 
				if snake.all_positions[#snake.all_positions].x == x and snake.all_positions[#snake.all_positions].y == y then  --if head
					love.graphics.setColor(1,1,1)
					love.graphics.draw(snake.sprites.head,pos[1],pos[2],snake.dto_r(snake.all_positions[#snake.all_positions].direction),1,1,19.5,18)
				else--if body 
					for c,b in pairs(snake.all_positions) do
						if b.x == x and b.y == y then 
							love.graphics.setColor(1,1,1)
							love.graphics.draw(snake.sprites.body,pos[1],pos[2],snake.dto_r(b.direction),1,1,19.5,18)
						end
					end
				end
				
			elseif v == 2 then -- if food draw red dot 
				love.graphics.setColor(1,1,1)
				love.graphics.draw(snake.sprites.food,pos[1],pos[2],0,1,1,19.5,18)
			end
		end
	end
end
return snake 